@@ -23,6 +23,7 @@
 #include "domain_capabilities.h"
 #include "domain_conf.h"
 #include "viralloc.h"
+#include "virstring.h"
 
 #define VIR_FROM_THIS VIR_FROM_CAPABILITIES
 
@@ -42,6 +43,7 @@ VIR_ENUM_IMPL(virDomainCapsFeature,
               "backup",
               "async-teardown",
               "s390-pv",
+              "ps2",
 );
 
 static virClass *virDomainCapsClass;
@@ -73,6 +75,7 @@ virSEVCapabilitiesFree(virSEVCapability *cap)
 
     g_free(cap->pdh);
     g_free(cap->cert_chain);
+    g_free(cap->cpu0_id);
     g_free(cap);
 }
 
@@ -120,6 +123,7 @@ virDomainCapsCPUModelsDispose(void *obj)
         g_free(cpuModels->models[i].name);
         g_strfreev(cpuModels->models[i].blockers);
         g_free(cpuModels->models[i].vendor);
+        g_free(cpuModels->models[i].canonical);
     }
 
     g_free(cpuModels->models);
@@ -182,7 +186,8 @@ virDomainCapsCPUModelsCopy(virDomainCapsCPUModels *old)
                                   old->models[i].usable,
                                   old->models[i].blockers,
                                   old->models[i].deprecated,
-                                  old->models[i].vendor);
+                                  old->models[i].vendor,
+                                  old->models[i].canonical);
     }
 
     return cpuModels;
@@ -195,7 +200,8 @@ virDomainCapsCPUModelsAdd(virDomainCapsCPUModels *cpuModels,
                           virDomainCapsCPUUsable usable,
                           char **blockers,
                           bool deprecated,
-                          const char *vendor)
+                          const char *vendor,
+                          const char *canonical)
 {
     virDomainCapsCPUModel *cpu;
 
@@ -210,6 +216,7 @@ virDomainCapsCPUModelsAdd(virDomainCapsCPUModels *cpuModels,
     cpu->blockers = g_strdupv(blockers);
     cpu->deprecated = deprecated;
     cpu->vendor = g_strdup(vendor);
+    cpu->canonical = g_strdup(canonical);
 }
 
 
@@ -231,6 +238,41 @@ virDomainCapsCPUModelsGet(virDomainCapsCPUModels *cpuModels,
 }
 
 
+static int
+virDomainCapsCPUModelsCompare(const void *m1,
+                              const void *m2,
+                              void *opaque G_GNUC_UNUSED)
+{
+    const virDomainCapsCPUModel *model1 = m1;
+    const virDomainCapsCPUModel *model2 = m2;
+
+    return strcmp(model1->name, model2->name);
+}
+
+
+void
+virDomainCapsCPUModelsSort(virDomainCapsCPUModels *cpuModels)
+{
+    size_t i;
+
+    g_qsort_with_data(cpuModels->models, cpuModels->nmodels,
+                      sizeof(*cpuModels->models),
+                      virDomainCapsCPUModelsCompare, NULL);
+
+    for (i = 0; i < cpuModels->nmodels; i++) {
+        virDomainCapsCPUModel *model = cpuModels->models + i;
+
+        if (!model->blockers)
+            continue;
+
+        g_qsort_with_data(model->blockers, g_strv_length(model->blockers),
+                          sizeof(*model->blockers),
+                          virStringSortCompare, NULL);
+        virStringListRemoveDuplicates(&model->blockers);
+    }
+}
+
+
 int
 virDomainCapsEnumSet(virDomainCapsEnum *capsEnum,
                      const char *capsEnumName,
@@ -393,7 +435,23 @@ virDomainCapsCPUCustomFormat(virBuffer *buf,
         else
             virBufferAddLit(buf, " vendor='unknown'");
 
+        if (model->canonical)
+            virBufferAsprintf(buf, " canonical='%s'", model->canonical);
+
         virBufferAsprintf(buf, ">%s</model>\n", model->name);
+
+        if (model->blockers) {
+            char **blocker;
+
+            virBufferAsprintf(buf, "<blockers model='%s'>\n", model->name);
+            virBufferAdjustIndent(buf, 2);
+
+            for (blocker = model->blockers; *blocker; blocker++)
+                virBufferAsprintf(buf, "<feature name='%s'/>\n", *blocker);
+
+            virBufferAdjustIndent(buf, -2);
+            virBufferAddLit(buf, "</blockers>\n");
+        }
     }
 
     virBufferAdjustIndent(buf, -2);
@@ -619,6 +677,18 @@ virDomainCapsDeviceNetFormat(virBuffer *buf,
 }
 
 
+static void
+virDomainCapsDevicePanicFormat(virBuffer *buf,
+                               const virDomainCapsDevicePanic *panic)
+{
+    FORMAT_PROLOGUE(panic);
+
+    ENUM_PROCESS(panic, model, virDomainPanicModelTypeToString);
+
+    FORMAT_EPILOGUE(panic);
+}
+
+
 /**
  * virDomainCapsFeatureGICFormat:
  * @buf: target buffer
@@ -800,6 +870,7 @@ virDomainCapsFormat(const virDomainCaps *caps)
     virDomainCapsDeviceChannelFormat(&buf, &caps->channel);
     virDomainCapsDeviceCryptoFormat(&buf, &caps->crypto);
     virDomainCapsDeviceNetFormat(&buf, &caps->net);
+    virDomainCapsDevicePanicFormat(&buf, &caps->panic);
 
     virBufferAdjustIndent(&buf, -2);
     virBufferAddLit(&buf, "</devices>\n");
