@@ -683,16 +683,9 @@ nodeDeviceObjFormatAddress(virNodeDeviceObj *obj)
             break;
             }
 
-        case VIR_NODE_DEV_CAP_CSS_DEV: {
-            virCCWDeviceAddress ccw_addr = {
-                .cssid = caps->data.ccw_dev.cssid,
-                .ssid = caps->data.ccw_dev.ssid,
-                .devno = caps->data.ccw_dev.devno
-            };
-
-            addr = virCCWDeviceAddressAsString(&ccw_addr);
+        case VIR_NODE_DEV_CAP_CSS_DEV:
+            addr = virCCWDeviceAddressAsString(caps->data.ccw_dev.dev_addr);
             break;
-            }
 
         case VIR_NODE_DEV_CAP_AP_MATRIX:
             addr = g_strdup(caps->data.ap_matrix.addr);
@@ -702,6 +695,10 @@ nodeDeviceObjFormatAddress(virNodeDeviceObj *obj)
             addr = g_strdup(caps->data.mdev_parent.address);
             break;
 
+        case VIR_NODE_DEV_CAP_CCWGROUP_DEV:
+            addr = virCCWDeviceAddressAsString(caps->data.ccwgroup_dev.address);
+            break;
+
         case VIR_NODE_DEV_CAP_SYSTEM:
         case VIR_NODE_DEV_CAP_USB_DEV:
         case VIR_NODE_DEV_CAP_USB_INTERFACE:
@@ -720,6 +717,7 @@ nodeDeviceObjFormatAddress(virNodeDeviceObj *obj)
         case VIR_NODE_DEV_CAP_AP_CARD:
         case VIR_NODE_DEV_CAP_AP_QUEUE:
         case VIR_NODE_DEV_CAP_VPD:
+        case VIR_NODE_DEV_CAP_CCWGROUP_MEMBER:
         case VIR_NODE_DEV_CAP_LAST:
             break;
         }
@@ -746,14 +744,14 @@ nodeDeviceGetMdevctlCommand(virNodeDeviceDef *def,
     case MDEVCTL_CMD_CREATE:
         /* now is the time to make sure "create" is replaced with "start" on
          * mdevctl cmdline */
-        cmd = virCommandNewArgList(MDEVCTL, "start", NULL);
+        cmd = virCommandNewArgList("mdevctl", "start", NULL);
         break;
     case MDEVCTL_CMD_STOP:
     case MDEVCTL_CMD_START:
     case MDEVCTL_CMD_DEFINE:
     case MDEVCTL_CMD_UNDEFINE:
     case MDEVCTL_CMD_MODIFY:
-        cmd = virCommandNewArgList(MDEVCTL, subcommand, NULL);
+        cmd = virCommandNewArgList("mdevctl", subcommand, NULL);
         break;
     case MDEVCTL_CMD_LAST:
     default:
@@ -774,8 +772,6 @@ nodeDeviceGetMdevctlCommand(virNodeDeviceDef *def,
         }
 
         if (nodeDeviceDefToMdevctlConfig(def, &inbuf, true) < 0) {
-            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
-                           _("couldn't convert node device def to mdevctl JSON"));
             return NULL;
         }
 
@@ -905,7 +901,7 @@ nodeDeviceGetMdevctlModifySupportCheck(void)
     g_autoptr(virCommand) cmd = NULL;
     const char *subcommand = virMdevctlCommandTypeToString(MDEVCTL_CMD_MODIFY);
 
-    cmd = virCommandNewArgList(MDEVCTL,
+    cmd = virCommandNewArgList("mdevctl",
                                subcommand,
                                "--defined",
                                "--live",
@@ -1134,7 +1130,7 @@ nodeDeviceGetMdevctlSetAutostartCommand(virNodeDeviceDef *def,
                                         bool autostart,
                                         char **errmsg)
 {
-    virCommand *cmd = virCommandNewArgList(MDEVCTL,
+    virCommand *cmd = virCommandNewArgList("mdevctl",
                                            "modify",
                                            "--uuid",
                                            def->caps->data.mdev.uuid,
@@ -1180,7 +1176,7 @@ nodeDeviceGetMdevctlListCommand(bool defined,
                                 char **output,
                                 char **errmsg)
 {
-    virCommand *cmd = virCommandNewArgList(MDEVCTL,
+    virCommand *cmd = virCommandNewArgList("mdevctl",
                                            "list",
                                            "--dumpjson",
                                            NULL);
@@ -1421,10 +1417,11 @@ nodeDeviceDestroy(virNodeDevicePtr device)
             goto cleanup;
 
         /* Because we're about to release the lock and thus run into a race
-         * possibility (however improbable) with a udevAddOneDevice change
-         * event which would essentially free the existing @def (obj->def) and
-         * replace it with something new, we need to grab the parent field
-         * and then find the parent obj in order to manage the vport */
+         * possibility (however improbable) with a
+         * processNodeDeviceAddAndChangeEvent change event which would
+         * essentially free the existing @def (obj->def) and replace it with
+         * something new, we need to grab the parent field and then find the
+         * parent obj in order to manage the vport */
         parent = g_strdup(def->parent);
 
         virNodeDeviceObjEndAPI(&obj);
@@ -1887,7 +1884,7 @@ removeMissingPersistentMdev(virNodeDeviceObj *obj,
 
 
 int
-nodeDeviceUpdateMediatedDevices(void)
+nodeDeviceUpdateMediatedDevices(virNodeDeviceDriverState *node_driver)
 {
     g_autofree virNodeDeviceDef **defs = NULL;
     g_autofree virNodeDeviceDef **act_defs = NULL;
@@ -1897,8 +1894,8 @@ nodeDeviceUpdateMediatedDevices(void)
     virMdevctlForEachData data = { 0, };
     size_t i;
 
-    if (!(mdevctl = virFindFileInPath(MDEVCTL))) {
-        VIR_DEBUG(MDEVCTL " not found. Skipping update of mediated devices.");
+    if (!(mdevctl = virFindFileInPath("mdevctl"))) {
+        VIR_DEBUG("'mdevctl' not found. Skipping update of mediated devices.");
         return 0;
     }
 
@@ -1911,7 +1908,7 @@ nodeDeviceUpdateMediatedDevices(void)
     /* Any mdevs that were previously defined but were not returned in the
      * latest mdevctl query should be removed from the device list */
     data.defs = defs;
-    virNodeDeviceObjListForEachRemove(driver->devs,
+    virNodeDeviceObjListForEachRemove(node_driver->devs,
                                       removeMissingPersistentMdev, &data);
 
     for (i = 0; i < data.ndefs; i++)
@@ -2018,6 +2015,19 @@ nodeDeviceDefCopyFromMdevctl(virNodeDeviceDef *dst,
 }
 
 
+/* A mediated device definition contains data from mdevctl about the active
+ * device. When the device is deactivated the active configuration data needs
+ * to be removed. */
+void
+nodeDeviceDefResetMdevActiveConfig(virNodeDeviceDef *def)
+{
+    if (def->caps->data.type != VIR_NODE_DEV_CAP_MDEV)
+        return;
+
+    virMediatedDeviceConfigClear(&def->caps->data.mdev.active_config);
+}
+
+
 int
 nodeDeviceSetAutostart(virNodeDevice *device,
                        int autostart)
@@ -2184,6 +2194,8 @@ int nodeDeviceDefValidate(virNodeDeviceDef *def,
             case VIR_NODE_DEV_CAP_AP_QUEUE:
             case VIR_NODE_DEV_CAP_AP_MATRIX:
             case VIR_NODE_DEV_CAP_VPD:
+            case VIR_NODE_DEV_CAP_CCWGROUP_DEV:
+            case VIR_NODE_DEV_CAP_CCWGROUP_MEMBER:
             case VIR_NODE_DEV_CAP_LAST:
                 break;
         }
@@ -2361,7 +2373,7 @@ nodeDeviceUpdate(virNodeDevice *device,
  cleanup:
     virNodeDeviceObjEndAPI(&obj);
     if (updated)
-        nodeDeviceUpdateMediatedDevices();
+        nodeDeviceUpdateMediatedDevices(driver);
 
     return ret;
 }
