@@ -469,6 +469,7 @@ typedef enum {
     QEMU_PROCESS_EVENT_GUESTPANIC,
     QEMU_PROCESS_EVENT_DEVICE_DELETED,
     QEMU_PROCESS_EVENT_NETDEV_STREAM_DISCONNECTED,
+    QEMU_PROCESS_EVENT_NETDEV_VHOST_USER_DISCONNECTED,
     QEMU_PROCESS_EVENT_NIC_RX_FILTER_CHANGED,
     QEMU_PROCESS_EVENT_SERIAL_CHANGED,
     QEMU_PROCESS_EVENT_JOB_STATUS_CHANGE,
@@ -690,14 +691,12 @@ int qemuDomainMomentDiscardAll(void *payload,
                                const char *name,
                                void *data);
 
-void qemuDomainRemoveInactive(virQEMUDriver *driver,
-                              virDomainObj *vm,
+void qemuDomainRemoveInactive(virDomainObj *vm,
                               virDomainUndefineFlagsValues flags,
-                              bool outgoingMigration);
+                              bool migration);
 
 void
-qemuDomainRemoveInactiveLocked(virQEMUDriver *driver,
-                               virDomainObj *vm);
+qemuDomainRemoveInactiveLocked(virDomainObj *vm);
 
 void qemuDomainSetFakeReboot(virDomainObj *vm,
                              bool value);
@@ -757,11 +756,6 @@ int qemuDomainStorageSourceAccessAllow(virQEMUDriver *driver,
                                        bool newSource,
                                        bool chainTop);
 
-int qemuDomainPrepareStorageSourceBlockdevNodename(virDomainDiskDef *disk,
-                                                   virStorageSource *src,
-                                                   const char *nodenameprefix,
-                                                   qemuDomainObjPrivate *priv,
-                                                   virQEMUDriverConfig *cfg);
 int qemuDomainPrepareStorageSourceBlockdev(virDomainDiskDef *disk,
                                            virStorageSource *src,
                                            qemuDomainObjPrivate *priv,
@@ -1173,3 +1167,7 @@ qemuDomainCheckCPU(virArch arch,
                    virCPUDef *cpu,
                    virQEMUCapsHostCPUType compatCPU,
                    bool failIncompatible);
+
+bool
+qemuDomainMachineSupportsFloppy(const char *machine,
+                                virQEMUCaps *qemuCaps);
