@@ -206,9 +206,11 @@ typedef void (*qemuMonitorDomainWatchdogCallback)(qemuMonitor *mon,
                                                   int action);
 typedef void (*qemuMonitorDomainIOErrorCallback)(qemuMonitor *mon,
                                                  virDomainObj *vm,
-                                                 const char *diskAlias,
+                                                 const char *device,
+                                                 const char *qompath,
                                                  const char *nodename,
                                                  int action,
+                                                 bool nospace,
                                                  const char *reason);
 typedef void (*qemuMonitorDomainGraphicsCallback)(qemuMonitor *mon,
                                                   virDomainObj *vm,
@@ -253,6 +255,9 @@ typedef void (*qemuMonitorDomainDeviceUnplugErrCallback)(qemuMonitor *mon,
 typedef void (*qemuMonitorDomainNetdevStreamDisconnectedCallback)(qemuMonitor *mon,
                                                                   virDomainObj *vm,
                                                                   const char *devAlias);
+typedef void (*qemuMonitorDomainNetdevVhostUserDisconnectedCallback)(qemuMonitor *mon,
+                                                                     virDomainObj *vm,
+                                                                     const char *devAlias);
 typedef void (*qemuMonitorDomainNicRxFilterChangedCallback)(qemuMonitor *mon,
                                                             virDomainObj *vm,
                                                             const char *devAlias);
@@ -401,6 +406,7 @@ struct _qemuMonitorCallbacks {
     qemuMonitorDomainMemoryDeviceSizeChange domainMemoryDeviceSizeChange;
     qemuMonitorDomainDeviceUnplugErrCallback domainDeviceUnplugError;
     qemuMonitorDomainNetdevStreamDisconnectedCallback domainNetdevStreamDisconnected;
+    qemuMonitorDomainNetdevVhostUserDisconnectedCallback domainNetdevVhostUserDisconnected;
 };
 
 qemuMonitor *qemuMonitorOpen(virDomainObj *vm,
@@ -430,7 +436,7 @@ int qemuMonitorSetLink(qemuMonitor *mon,
 /* These APIs are for use by the internal Text/JSON monitor impl code only */
 char *qemuMonitorNextCommandID(qemuMonitor *mon);
 int qemuMonitorSend(qemuMonitor *mon,
-                    qemuMonitorMessage *msg) G_NO_INLINE;
+                    qemuMonitorMessage *msg) ATTRIBUTE_MOCKABLE;
 int qemuMonitorUpdateVideoMemorySize(qemuMonitor *mon,
                                      virDomainVideoDef *video,
                                      const char *videoName)
@@ -450,9 +456,11 @@ void qemuMonitorEmitResume(qemuMonitor *mon);
 void qemuMonitorEmitRTCChange(qemuMonitor *mon, long long offset);
 void qemuMonitorEmitWatchdog(qemuMonitor *mon, int action);
 void qemuMonitorEmitIOError(qemuMonitor *mon,
-                            const char *diskAlias,
+                            const char *device,
+                            const char *qompath,
                             const char *nodename,
                             int action,
+                            bool nospace,
                             const char *reason);
 void qemuMonitorEmitGraphics(qemuMonitor *mon,
                              int phase,
@@ -486,6 +494,8 @@ void qemuMonitorEmitDeviceUnplugErr(qemuMonitor *mon,
                                     const char *devAlias);
 void qemuMonitorEmitNetdevStreamDisconnected(qemuMonitor *mon,
                                              const char *devAlias);
+void qemuMonitorEmitNetdevVhostUserDisconnected(qemuMonitor *mon,
+                                                const char *devAlias);
 void qemuMonitorEmitNicRxFilterChanged(qemuMonitor *mon,
                                        const char *devAlias);
 void qemuMonitorEmitSerialChange(qemuMonitor *mon,
@@ -558,7 +568,6 @@ typedef enum {
 VIR_ENUM_DECL(qemuMonitorVMStatus);
 int qemuMonitorVMStatusToPausedReason(const char *status);
 
-int qemuMonitorCheck(qemuMonitor *mon);
 int qemuMonitorGetStatus(qemuMonitor *mon,
                          bool *running,
                          virDomainPausedReason *reason)
@@ -701,6 +710,13 @@ struct _qemuBlockNamedNodeDataBitmap {
     unsigned long long granularity;
 };
 
+
+typedef struct _qemuBlockNamedNodeDataSnapshot qemuBlockNamedNodeDataSnapshot;
+struct _qemuBlockNamedNodeDataSnapshot {
+    bool vmstate;
+};
+
+
 typedef struct _qemuBlockNamedNodeData qemuBlockNamedNodeData;
 struct _qemuBlockNamedNodeData {
     unsigned long long capacity;
@@ -709,6 +725,10 @@ struct _qemuBlockNamedNodeData {
     qemuBlockNamedNodeDataBitmap **bitmaps;
     size_t nbitmaps;
 
+    /* hash table indexed by snapshot name containing data about snapshots
+     * (qemuBlockNamedNodeDataSnapshot) */
+    GHashTable *snapshots;
+
     /* the cluster size of the image is valid only when > 0 */
     unsigned long long clusterSize;
 
@@ -845,6 +865,11 @@ int qemuMonitorMigrateToFd(qemuMonitor *mon,
                            unsigned int flags,
                            int fd);
 
+int qemuMonitorMigrateToFdSet(virDomainObj *vm,
+                              unsigned int flags,
+                              int *fd,
+                              int *directFd);
+
 int qemuMonitorMigrateToHost(qemuMonitor *mon,
                              unsigned int flags,
                              const char *protocol,
@@ -868,8 +893,7 @@ int qemuMonitorQueryDump(qemuMonitor *mon,
 
 int qemuMonitorDumpToFd(qemuMonitor *mon,
                         int fd,
-                        const char *dumpformat,
-                        bool detach);
+                        const char *dumpformat);
 
 int qemuMonitorGraphicsRelocate(qemuMonitor *mon,
                                 int type,
@@ -936,10 +960,6 @@ void qemuMonitorChardevInfoFree(void *data);
 int qemuMonitorGetChardevInfo(qemuMonitor *mon,
                               GHashTable **retinfo);
 
-int qemuMonitorAttachPCIDiskController(qemuMonitor *mon,
-                                       const char *bus,
-                                       virPCIDeviceAddress *guestAddr);
-
 int qemuMonitorAddDeviceProps(qemuMonitor *mon,
                               virJSONValue **props);
 
@@ -961,9 +981,6 @@ int qemuMonitorDelObject(qemuMonitor *mon,
                          const char *objalias,
                          bool report_error);
 
-int qemuMonitorCreateSnapshot(qemuMonitor *mon, const char *name);
-int qemuMonitorDeleteSnapshot(qemuMonitor *mon, const char *name);
-
 int qemuMonitorTransaction(qemuMonitor *mon, virJSONValue **actions)
     ATTRIBUTE_NONNULL(2);
 int qemuMonitorBlockdevMirror(qemuMonitor *mon,
@@ -971,6 +988,7 @@ int qemuMonitorBlockdevMirror(qemuMonitor *mon,
                               bool persistjob,
                               const char *device,
                               const char *target,
+                              const char *replaces,
                               unsigned long long bandwidth,
                               unsigned int granularity,
                               unsigned long long buf_size,
@@ -1064,6 +1082,15 @@ int qemuMonitorGetBlockIoThrottle(qemuMonitor *mon,
                                   const char *qdevid,
                                   virDomainBlockIoTuneInfo *reply);
 
+int
+qemuMonitorThrottleGroupLimits(virJSONValue *limits,
+                               const virDomainThrottleGroupDef *group);
+
+int
+qemuMonitorUpdateThrottleGroup(qemuMonitor *mon,
+                               const char *qomid,
+                               virDomainBlockIoTuneInfo *info);
+
 int qemuMonitorSystemWakeup(qemuMonitor *mon);
 
 int qemuMonitorGetVersion(qemuMonitor *mon,
@@ -1143,6 +1170,7 @@ struct _qemuMonitorCPUModelInfo {
     char *name;
     size_t nprops;
     qemuMonitorCPUProperty *props;
+    GStrv deprecated_props;
     bool migratability;
 };
 
@@ -1195,11 +1223,6 @@ int qemuMonitorNBDServerStart(qemuMonitor *mon,
                               const virStorageNetHostDef *server,
                               const char *tls_alias)
     ATTRIBUTE_NONNULL(2);
-int qemuMonitorNBDServerAdd(qemuMonitor *mon,
-                            const char *deviceID,
-                            const char *export,
-                            bool writable,
-                            const char *bitmap);
 int qemuMonitorNBDServerStop(qemuMonitor *mon);
 
 int qemuMonitorBlockExportAdd(qemuMonitor *mon,
@@ -1287,7 +1310,8 @@ int qemuMonitorGetMemoryDeviceInfo(qemuMonitor *mon,
     ATTRIBUTE_NONNULL(2);
 
 int qemuMonitorMigrateIncoming(qemuMonitor *mon,
-                               const char *uri);
+                               const char *uri,
+                               virTristateBool exitOnError);
 
 int qemuMonitorMigrateStartPostCopy(qemuMonitor *mon);
 
@@ -1618,3 +1642,29 @@ int
 qemuMonitorDisplayReload(qemuMonitor *mon,
                          const char *type,
                          bool tlsCerts);
+
+
+int
+qemuMonitorSnapshotSave(qemuMonitor *mon,
+                        const char *jobname,
+                        const char *snapshotname,
+                        const char *vmstate_disk,
+                        const char **disks);
+
+int
+qemuMonitorSnapshotLoad(qemuMonitor *mon,
+                        const char *jobname,
+                        const char *snapshotname,
+                        const char *vmstate_disk,
+                        const char **disks);
+
+int
+qemuMonitorSnapshotDelete(qemuMonitor *mon,
+                          const char *jobname,
+                          const char *snapshotname,
+                          const char **disks);
+
+int
+qemuMonitorBlockdevSetActive(qemuMonitor *mon,
+                             const char *nodename,
+                             bool active);
