@@ -24,6 +24,7 @@ static virCPUDef *cpuHaswell;
 static virCPUDef *cpuPower8;
 static virCPUDef *cpuPower9;
 static virCPUDef *cpuPower10;
+static virCPUDef *cpuPower11;
 
 char *
 virFindFileInPath(const char *file)
@@ -52,6 +53,12 @@ bool virTPMHasSwtpm(void)
 }
 
 
+/* Enough to tell capabilities code that qemu-rdp is usable */
+bool qemuRdpAvailable(const char *helper G_GNUC_UNUSED)
+{
+    return true;
+}
+
 
 bool
 virTPMSwtpmSetupCapsGet(virTPMSwtpmSetupFeature cap)
@@ -71,6 +78,7 @@ virTPMSwtpmSetupCapsGet(virTPMSwtpmSetupFeature cap)
     case VIR_TPM_SWTPM_SETUP_FEATURE_CMDARG_CREATE_CONFIG_FILES:
     case VIR_TPM_SWTPM_SETUP_FEATURE_TPM12_NOT_NEED_ROOT:
     case VIR_TPM_SWTPM_SETUP_FEATURE_CMDARG_RECONFIGURE_PCR_BANKS:
+    case VIR_TPM_SWTPM_SETUP_FEATURE_CMDARG_PROFILE:
     case VIR_TPM_SWTPM_SETUP_FEATURE_LAST:
         break;
     }
@@ -159,6 +167,7 @@ qemuTestGetCPUDef(qemuTestCPUDef d)
     case QEMU_CPU_DEF_POWER8: return cpuPower8;
     case QEMU_CPU_DEF_POWER9: return cpuPower9;
     case QEMU_CPU_DEF_POWER10: return cpuPower10;
+    case QEMU_CPU_DEF_POWER11: return cpuPower11;
     }
 
     return NULL;
@@ -244,6 +253,7 @@ void qemuTestDriverFree(virQEMUDriver *driver)
     virCPUDefFree(cpuPower8);
     virCPUDefFree(cpuPower9);
     virCPUDefFree(cpuPower10);
+    virCPUDefFree(cpuPower11);
 }
 
 
@@ -296,6 +306,7 @@ int qemuTestDriverInit(virQEMUDriver *driver)
     cpuPower8 = virCPUDefCopy(&cpuPower8Data);
     cpuPower9 = virCPUDefCopy(&cpuPower9Data);
     cpuPower10 = virCPUDefCopy(&cpuPower10Data);
+    cpuPower11 = virCPUDefCopy(&cpuPower11Data);
 
     if (virMutexInit(&driver->lock) < 0)
         return -1;
@@ -329,6 +340,8 @@ int qemuTestDriverInit(virQEMUDriver *driver)
     cfg->passtStateDir = g_strdup("/var/run/libvirt/qemu/passt");
     VIR_FREE(cfg->dbusStateDir);
     cfg->dbusStateDir = g_strdup("/var/run/libvirt/qemu/dbus");
+    VIR_FREE(cfg->rdpStateDir);
+    cfg->rdpStateDir = g_strdup("/var/run/libvirt/qemu/rdp");
 
     if (!g_mkdtemp(statedir)) {
         fprintf(stderr, "Cannot create fake stateDir");
@@ -379,6 +392,8 @@ int qemuTestDriverInit(virQEMUDriver *driver)
     cfg->vncTLSx509certdir = g_strdup("/etc/pki/libvirt-vnc");
     VIR_FREE(cfg->spiceTLSx509certdir);
     cfg->spiceTLSx509certdir = g_strdup("/etc/pki/libvirt-spice");
+    VIR_FREE(cfg->rdpTLSx509certdir);
+    cfg->rdpTLSx509certdir = g_strdup("/etc/pki/libvirt-rdp");
     VIR_FREE(cfg->chardevTLSx509certdir);
     cfg->chardevTLSx509certdir = g_strdup("/etc/pki/libvirt-chardev");
     VIR_FREE(cfg->vxhsTLSx509certdir);
@@ -703,11 +718,13 @@ testQemuInfoSetArgs(testQemuInfo *info,
             virStorageSourceFDTuple *new = virStorageSourceFDTupleNew();
             const char *fdname = va_arg(argptr, char *);
             VIR_AUTOCLOSE fakefd = open("/dev/zero", O_RDWR);
+            bool writable = va_arg(argptr, int);
             size_t i;
 
             new->nfds = va_arg(argptr, unsigned int);
             new->fds = g_new0(int, new->nfds);
             new->testfds = g_new0(int, new->nfds);
+            new->writable = writable;
 
             for (i = 0; i < new->nfds; i++) {
                 new->testfds[i] = va_arg(argptr, unsigned int);
@@ -751,6 +768,9 @@ testQemuInfoSetArgs(testQemuInfo *info,
         if (info->args.invalidarg)
             break;
     }
+
+    if (!info->args.capsvariant)
+        info->args.capsvariant = "";
 }
 
 
@@ -910,9 +930,6 @@ testQemuInfoInitArgs(testQemuInfo *info)
 
     info->args.newargs = false;
 
-    if (!info->args.capsvariant)
-        info->args.capsvariant = "";
-
     if (info->args.invalidarg) {
         fprintf(stderr, "Invalid argument encountered by 'testQemuInfoSetArgs'\n");
         return -1;
