@@ -45,7 +45,7 @@ static const vshCmdInfo info_capabilities = {
 static const vshCmdOptDef opts_capabilities[] = {
     {.name = "xpath",
      .type = VSH_OT_STRING,
-     .completer = virshCompleteEmpty,
+     .completer = vshCompleteEmpty,
      .help = N_("xpath expression to filter the XML document")
     },
     {.name = "wrap",
@@ -107,13 +107,17 @@ static const vshCmdOptDef opts_domcapabilities[] = {
     },
     {.name = "xpath",
      .type = VSH_OT_STRING,
-     .completer = virshCompleteEmpty,
+     .completer = vshCompleteEmpty,
      .help = N_("xpath expression to filter the XML document")
     },
     {.name = "wrap",
      .type = VSH_OT_BOOL,
      .help = N_("wrap xpath results in an common root element"),
     },
+    {.name = "disable-deprecated-features",
+     .type = VSH_OT_BOOL,
+     .help = N_("report host CPU model with deprecated features disabled"),
+    },
     {.name = NULL}
 };
 
@@ -126,10 +130,13 @@ cmdDomCapabilities(vshControl *ctl, const vshCmd *cmd)
     const char *arch = NULL;
     const char *machine = NULL;
     const char *xpath = NULL;
-    const unsigned int flags = 0; /* No flags so far */
+    unsigned int flags = 0;
     bool wrap = vshCommandOptBool(cmd, "wrap");
     virshControl *priv = ctl->privData;
 
+    if (vshCommandOptBool(cmd, "disable-deprecated-features"))
+        flags |= VIR_CONNECT_GET_DOMAIN_CAPABILITIES_DISABLE_DEPRECATED_FEATURES;
+
     if (vshCommandOptString(ctl, cmd, "virttype", &virttype) < 0 ||
         vshCommandOptString(ctl, cmd, "emulatorbin", &emulatorbin) < 0 ||
         vshCommandOptString(ctl, cmd, "arch", &arch) < 0 ||
@@ -448,7 +455,7 @@ cmdFreepages(vshControl *ctl, const vshCmd *cmd)
 
         if (rv == 0) {
             vshError(ctl,
-                     "Could not get count of free %uKiB pages, no data returned",
+                     _("Could not get count of free %1$uKiB pages, no data returned"),
                      *pagesize);
             goto cleanup;
         }
@@ -1086,7 +1093,8 @@ cmdURI(vshControl *ctl, const vshCmd *cmd G_GNUC_UNUSED)
  */
 static char **
 vshExtractCPUDefXMLs(vshControl *ctl,
-                     const char *xmlFile)
+                     const char *xmlFile,
+                     bool hostCPUWarning)
 {
     g_auto(GStrv) cpus = NULL;
     g_autofree char *buffer = NULL;
@@ -1130,19 +1138,26 @@ vshExtractCPUDefXMLs(vshControl *ctl,
     cpus = g_new0(char *, n + 1);
 
     for (i = 0; i < n; i++) {
+        ctxt->node = nodes[i];
+
         /* If the user provided domain capabilities XML, we need to replace
          * <mode ...> element with <cpu>. */
         if (xmlStrEqual(nodes[i]->name, BAD_CAST "mode")) {
             xmlNodeSetName(nodes[i], (const xmlChar *)"cpu");
             while (nodes[i]->properties) {
                 if (xmlRemoveProp(nodes[i]->properties) < 0) {
-                    vshError(ctl,
+                    vshError(ctl, "%s",
                              _("Cannot extract CPU definition from domain capabilities XML"));
                     return NULL;
                 }
             }
         }
 
+        if (hostCPUWarning &&
+            virXPathBoolean("boolean(./feature[not(@policy)])", ctxt) == 1) {
+            vshWarn(ctl, "%s", _("using host CPU definition as input may provide incorrect results"));
+        }
+
         if (!(cpus[i] = virXMLNodeToString(xml, nodes[i]))) {
             vshSaveLibvirtError();
             return NULL;
@@ -1192,7 +1207,7 @@ cmdCPUCompare(vshControl *ctl, const vshCmd *cmd)
     if (vshCommandOptString(ctl, cmd, "file", &from) < 0)
         return false;
 
-    if (!(cpus = vshExtractCPUDefXMLs(ctl, from)))
+    if (!(cpus = vshExtractCPUDefXMLs(ctl, from, false)))
         return false;
 
     result = virConnectCompareCPU(priv->conn, cpus[0], flags);
@@ -1202,7 +1217,6 @@ cmdCPUCompare(vshControl *ctl, const vshCmd *cmd)
         vshPrint(ctl, _("CPU described in %1$s is incompatible with host CPU\n"),
                  from);
         return false;
-        break;
 
     case VIR_CPU_COMPARE_IDENTICAL:
         vshPrint(ctl, _("CPU described in %1$s is identical to host CPU\n"),
@@ -1261,7 +1275,7 @@ cmdCPUBaseline(vshControl *ctl, const vshCmd *cmd)
     if (vshCommandOptString(ctl, cmd, "file", &from) < 0)
         return false;
 
-    if (!(list = vshExtractCPUDefXMLs(ctl, from)))
+    if (!(list = vshExtractCPUDefXMLs(ctl, from, false)))
         return false;
 
     if (!(result = virConnectBaselineCPU(priv->conn, (const char **)list,
@@ -1605,7 +1619,7 @@ cmdHypervisorCPUCompare(vshControl *ctl,
         vshCommandOptString(ctl, cmd, "machine", &machine) < 0)
         return false;
 
-    if (!(cpus = vshExtractCPUDefXMLs(ctl, from)))
+    if (!(cpus = vshExtractCPUDefXMLs(ctl, from, true)))
         return false;
 
     result = virConnectCompareHypervisorCPU(priv->conn, emulator, arch,
@@ -1617,7 +1631,6 @@ cmdHypervisorCPUCompare(vshControl *ctl,
                  _("CPU described in %1$s is incompatible with the CPU provided by hypervisor on the host\n"),
                  from);
         return false;
-        break;
 
     case VIR_CPU_COMPARE_IDENTICAL:
         vshPrint(ctl,
@@ -1654,7 +1667,7 @@ static const vshCmdOptDef opts_hypervisor_cpu_baseline[] = {
     {.name = "file",
      .type = VSH_OT_STRING,
      .unwanted_positional = true,
-     .completer = virshCompletePathLocalExisting,
+     .completer = vshCompletePathLocalExisting,
      .help = N_("file containing XML CPU descriptions"),
     },
     {.name = "virttype",
@@ -1729,7 +1742,7 @@ cmdHypervisorCPUBaseline(vshControl *ctl,
     VSH_ALTERNATIVE_OPTIONS_EXPR("file", from, "model", model);
 
     if (from) {
-        if (!(list = vshExtractCPUDefXMLs(ctl, from)))
+        if (!(list = vshExtractCPUDefXMLs(ctl, from, true)))
             return false;
     } else {
         list = g_new0(char *, 2);
@@ -1751,6 +1764,75 @@ cmdHypervisorCPUBaseline(vshControl *ctl,
 }
 
 
+/*
+ * "hypervisor-cpu-models" command
+ */
+static const vshCmdInfo info_hypervisor_cpu_models = {
+    .help = N_("Hypervisor reported CPU models"),
+    .desc = N_("Get the CPU models reported by the hypervisor."),
+};
+
+static const vshCmdOptDef opts_hypervisor_cpu_models[] = {
+    {.name = "virttype",
+     .type = VSH_OT_STRING,
+     .completer = virshDomainVirtTypeCompleter,
+     .help = N_("virtualization type (/domain/@type)"),
+    },
+    {.name = "emulator",
+     .type = VSH_OT_STRING,
+     .help = N_("path to emulator binary (/domain/devices/emulator)"),
+    },
+    {.name = "arch",
+     .type = VSH_OT_STRING,
+     .completer = virshArchCompleter,
+     .help = N_("CPU architecture (/domain/os/type/@arch)"),
+    },
+    {.name = "machine",
+     .type = VSH_OT_STRING,
+     .help = N_("machine type (/domain/os/type/@machine)"),
+    },
+    {.name = "all",
+     .type = VSH_OT_BOOL,
+     .help = N_("include all CPU models known to the hypervisor for the architecture")
+    },
+    {.name = NULL}
+};
+
+static bool
+cmdHypervisorCPUModelNames(vshControl *ctl,
+                           const vshCmd *cmd)
+{
+    g_autofree char *caps_xml = NULL;
+    const char *virttype = NULL;
+    const char *emulator = NULL;
+    const char *arch = NULL;
+    const char *machine = NULL;
+    const char *xpath = NULL;
+    virshControl *priv = ctl->privData;
+
+    if (vshCommandOptString(ctl, cmd, "virttype", &virttype) < 0 ||
+        vshCommandOptString(ctl, cmd, "emulator", &emulator) < 0 ||
+        vshCommandOptString(ctl, cmd, "arch", &arch) < 0 ||
+        vshCommandOptString(ctl, cmd, "machine", &machine) < 0)
+        return false;
+
+    if (vshCommandOptBool(cmd, "all"))
+        xpath = "//cpu//model[@usable]/text()";
+    else
+        xpath = "//cpu//model[@usable='yes']/text()";
+
+    caps_xml = virConnectGetDomainCapabilities(priv->conn, emulator, arch,
+                                               machine, virttype, 0);
+
+    if (!caps_xml) {
+        vshError(ctl, "%s", _("failed to get hypervisor CPU model names"));
+        return false;
+    }
+
+    return virshDumpXML(ctl, caps_xml, "domcapabilities", xpath, false);
+}
+
+
 const vshCmdDef hostAndHypervisorCmds[] = {
     {.name = "allocpages",
      .handler = cmdAllocpages,
@@ -1818,6 +1900,12 @@ const vshCmdDef hostAndHypervisorCmds[] = {
      .info = &info_hypervisor_cpu_compare,
      .flags = 0
     },
+    {.name = "hypervisor-cpu-models",
+     .handler = cmdHypervisorCPUModelNames,
+     .opts = opts_hypervisor_cpu_models,
+     .info = &info_hypervisor_cpu_models,
+     .flags = 0
+    },
     {.name = "maxvcpus",
      .handler = cmdMaxvcpus,
      .opts = opts_maxvcpus,
